db_passwords = {
    "service_registry": "ZzNNpxrbZGVvfJ8",
    "authorization": "hqZFUkuHxhekio3",
    "orchestrator": "KbgD2mTr8DQ4vtc",
    "event_handler": "gRLjXbqu9YwYhfK",
    "gateway": "LfiSM9DpGfDEP5g",
    "gatekeeper": "fbJKYzKhU5t8QtT",
    "system_registry": "Kh12Hhgaxzo7haf",
    "device_registry": "iooHU87hNGUalht",
    "onboarding_controller": "JKgh1as5f6oi7aV",
    "certificate_authority": "FsdG6Kgf9QpPfv2",
}
